{% extends "admin/base_site.html" %}
{% load i18n admin_urls static %}

{% block extrahead %}
    {{ block.super }}
    {{ media }}
    <script type="text/javascript" src="{% static 'admin/js/cancel.js' %}"></script>
    <script type="text/javascript" src="{% static 'djangocms_versioning/js/admin/versioning-actions.js' %}"></script>
{% endblock %}

{% block breadcrumbs %}{% endblock %}
{% block bodyclass %}{{ block.super }} app-{{ opts.app_label }} model-{{ opts.model_name }} delete-confirmation{% endblock %}

{% block content %}
<h1>{% block title %}{% translate "Revert Confirmation" %}{% endblock %}</h1>
<p>{% translate "Unpublishing will remove this version from live. Are you sure you want to unpublish?" %}</p>
<h3>{{ object_name }}</h3>
<h4>{% blocktrans %} Version number: {{ version_number }}{% endblocktrans %}</h4>
<div>
    {% for thing in extra_context.values %}
        <p>{{thing}}</p>
    {% endfor %}
</div>
<form action="" method="POST" class="js-close-sideframe">
    {% csrf_token %}
    <div class="submit-row">
        <input  class="button confirm-link"
                type="submit"
                value="{% translate "Yes, I'm sure" %}">
        <a href="{{ back_url }}">
            <input type="button"
                   class="button js-versioning-keep-sideframe"
                   value="{% translate 'No, take me back' %}">
        </a>
    </div>
</form>
{% endblock %}
